
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <limits.h>
#include <assert.h>
#include <iostream>
#include <string.h>
#include <sstream>
#include <ostream>

#include "InOut.h"
#include "QAP.h"
#include "ants.h"
#include "ls.h"
#include "utilities.h"
#include "kriging.h"

#include <Eigen/Dense>

long int n;          // number of cities in the instance to be solved 


struct problem instance;


double surrogate_tour_length(long int * x, std::string result){
/*    
      FUNCTION:      using the surrogate model generated, computes the objective function
                     of a permutation
      INPUT:         pointer to a permutation
      OUTPUT:        none
      (SIDE)EFFECTS: none
      COMMENTS:      
*/
  
  long int n_sol = solutions.rows();

  Eigen::RowVectorXd psi (n_sol);
  Eigen::VectorXd dista (n_sol);
     
  int aux=0;
  //Calculate distance
  for(int i=0; i < n_sol ; i++){
     aux=0;
     for(int j=0; j < n ; j++) {  
       if(solutions(i,j)!= x[j]) aux++;
     }
     dista(i) = (double) aux/n;
     //psi(i) = (double) exp(-theta*dista(i));
     psi(i) = (double) exp(-dmodeltheta*dista(i));
  }

  double f = psi * (psinv * yon);
  f  = f + mu;

  if(result=="ei"){
    Eigen::VectorXd eval_aux = evaluations.cast<double>().array() - f;
    eval_aux=eval_aux.array().square(); 
    double yerror = sqrt(eval_aux.sum());

    long int eval_min = evaluations.minCoeff();
    double t1 = ( eval_min - f) * phi( (eval_min -f )/ yerror );
    double t2_1 = (double) (1/sqrt(2*M_PI));
    double t2_2 = (double) (pow(eval_min - f, 2))/ pow(yerror,2);
    t2_2 = (double) exp( -((double) 1/2) * t2_2) ;
    double t2 = (double) yerror * t2_1 * t2_2;
    
    //double ei = log(t1 + t2 + std::numeric_limits< double >::min());
    double ei = t1 + t2 + std::numeric_limits< double >::min();
    ei = (evaluations(0)*100000)/ei;
    
    if(ei <= 0){
      std::cout << "Negative expected improvement. Setting expected improvement to a small number." << std::endl;   
      return(std::numeric_limits<double>::max());
    }
    return(ei);
  }else{
    return(f);
  }
  
  return(f); 
}

double surrogate_tour_length( VectorXl x, std::string result){
/*    
      FUNCTION:      using the surrogate model generated, computes the objective function
                     of a permutation
      INPUT:         permutation vector
      OUTPUT:        none
      (SIDE)EFFECTS: none
      COMMENTS:      
*/  
  long int n_sol = solutions.rows();

  Eigen::RowVectorXd psi (n_sol);
  Eigen::VectorXd dista (n_sol);
     
  int aux=0;
  //Calculate distance!
  for(int i=0; i < n_sol ; i++){
     aux=0;
     for(int j=0; j < n ; j++) {  
       if(solutions(i,j)!= x(j)) aux++;
     }
     dista(i) = (double) aux/n;
     psi(i) = (double) exp(-theta*dista(i));
     //psi(i) = (double) exp(-dmodeltheta*dista(i));
  }

  double f = psi * (psinv * yon);
  f  = f + mu;

  if(result=="ei"){
    Eigen::VectorXd eval_aux = evaluations.cast<double>().array() - f;
    eval_aux=eval_aux.array().square(); 
    double yerror = sqrt(eval_aux.sum());

    long int eval_min = evaluations.minCoeff();
    double t1 = ( eval_min - f) * phi( (eval_min -f )/ yerror );
    double t2_1 = (double) (1/sqrt(2*M_PI));
    double t2_2 = (double) (pow(eval_min - f, 2))/ pow(yerror,2);
    t2_2 = (double) exp( -((double) 1/2) * t2_2) ;
    double t2 = (double) yerror * t2_1 * t2_2;

    //double ei = log(t1 + t2 + std::numeric_limits< double >::min());
    double ei = t1 + t2 + std::numeric_limits< double >::min();
    ei = evaluations(0)/ei;
    if(ei <= 0){
      std::cout << "Negative expected improvement. Setting expected improvement to a small number." << std::endl;
      return(std::numeric_limits<double>::max());
    }
    return(ei);
  }else{
    return(f);
  }
  
  return(f);
}



long int compute_tour_length( long int *p ) {
/*    
      FUNCTION:      computes the objective function value of a permutation
      INPUT:         pointer to a permutation
      OUTPUT:        none
      (SIDE)EFFECTS: none
      COMMENTS:      Division by 2 has to be done if we have an asymmetric instance that has 
                     been converted into a symmetric one (indicated by make_symmetric_flag). 
		                 This is due to the particular way of doing this conversion.
*/
   long int  i, j;
   unsigned long  obj_f_value; /* must be unsigned, because with only "long int" we have an overflow
				                         on some few instances,  */
   obj_f_value = 0;
   for ( i = 0 ; i < n ; i++ ) {
     for ( j = 0 ; j < n ; j++ ) {
       obj_f_value += instance.distance[i][j] * instance.flow[p[i]][p[j]];
     }
   }
   VERYVERBOSE ( printf("objective function value = %lu \n\n",obj_f_value); )
   return obj_f_value;
}


long int compute_tour_length( VectorXl p  ) {
/*    
      FUNCTION:      computes the objective function value of a permutation
      INPUT:         permutation vector
      OUTPUT:        none
      (SIDE)EFFECTS: none
      COMMENTS:      Division by 2 has to be done if we have an asymmetric instance that has 
                     been converted into a symmetric one (indicated by make_symmetric_flag). 
		     This is due to the particular way of doing this conversion.
*/
   long int  i, j;
   unsigned long  obj_f_value; /* unsigned, because with only "long int" we have an overflow
				  on some few instances, for example, tai100b. This is because 
				  of making this instance symmetric (see make_matrix_symmetric) 
			       */
   obj_f_value = 0;
   for ( i = 0 ; i < n ; i++ ) {
     for ( j = 0 ; j < n ; j++ ) {
       obj_f_value += instance.distance[i][j] * instance.flow[p(i)][p(j)];
     }
   }
   VERYVERBOSE ( printf("objective function value = %lu \n\n",obj_f_value); )
   return obj_f_value;
}




void read_problem_size( FILE *input ) {
/* 
      FUNCTION:      read the dimension of the QAP instance
      INPUT:         a pointer to the input file 
      OUTPUT:        none
      (SIDE)EFFECTS: assigns n the instance dimension (= number of locations / objects)
*/
  if( fscanf(input, "%ld", &n) <= 0) {
    fprintf(stderr, "error reading qap size value in data file\n");
    exit(1);	
  }
  VERYVERBOSE ( printf("QAP instance size %ld\n",n); )
}



void read_best_known_value( FILE *input ) {
/* 
      FUNCTION:      read the best known objective function value of the QAP instance
      INPUT:         a pointer to the input file 
      OUTPUT:        none
      (SIDE)EFFECTS: assigns best_known the read value
*/
  long int i;

  if( fscanf(input, "%ld ", &i) < 0 ) {
    fprintf(stderr, "error reading best known solution in data file\n");
    exit(1);	
  }
  VERYVERBOSE ( printf(" Best known solution %ld\n",i); )
}



long int ** read_matrix( FILE *input, long int size )
/*    
      FUNCTION:      read a QAP instance matrix from the input file
      INPUT:         Pointer to input file, size of QAP instance
      OUTPUT:        pointer to matrix, has to be freed before program stops
      (SIDE)EFFECTS: allocates a memory of appropriate size for the matrix
*/
{
  long int     i, j;
  long int     **matrix;

  if((matrix = (long int **) malloc(sizeof(long int) * size * size +
		      sizeof(long int *) * size	 )) == NULL){
    fprintf(stderr,"Out of memory, exit.\n"); 
    exit(1);
  }
  for ( i = 0 ; i < size ; i++ ) {
    matrix[i] = (long int *)(matrix + size) + i*size;
    for ( j = 0  ; j < size ; j++ ) {
      if( fscanf(input, "%ld", &matrix[i][j]) < 0) {
	fprintf(stderr, "error reading (%ld, %ld) qap_distance in data file\n", i, j);
	exit(1);	
      }
    }
  }
  return matrix;
}


