

/***************************************************************************

    Program's name: egaco-qap

    Implementation of the EGO algorithm for the Quadratic assigment problem.
    Thee algorithm uses ACO (AS, ACS, EAS, RAS, MMAS, BWAS) to optimize the 
    surrogate model created by EGO. The implementation is based on the ACOTSP 
    software by Thomas Stuetzle.
    (http://iridia.ulb.ac.be/~mdorigo/ACO/aco-code/public-software.html).

    Version: 1.0
    File:    main.c
    Author:  Leslie Perez Caceres
    Check:   README and gpl.txt
    Copyright (C) 2015  Leslie Perez Caceres

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    email: leslie.perez.caeeres no@spam ulb.ac.be
    mail address: Universite Libre de Bruxelles
		  IRIDIA

		 Belgium

***************************************************************************/


#include <stdio.h>
#include <math.h>
#include <limits.h>
#include <assert.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <iostream>
#include <fstream>
#include <sstream>
#include <ostream>

#include "ants.h"
#include "utilities.h"
#include "InOut.h"
#include "QAP.h"
#include "timer.h"
#include "ls.h"
#include "adaptation.h"
#include "kriging.h"

#include <vector>
#include <Eigen/Dense>

using namespace Eigen;
using namespace std;

/*Model data variables*/
double loval, upval, dmodeltheta, theta, p, ssq, mu;
VectorXd yon;
MatrixXd psi;
MatrixXd psinv; 
MatrixXl solutions;
VectorXl evaluations;

MatrixXl all_solutions;
VectorXl all_evaluations;

/*EGO best*/
long int best_index, best_nevaluation,  best_fx = (long int) 9999999999999;
VectorXl best_solution;

long int termination_condition( void )
/*    
      FUNCTION:       checks whether termination condition is met 
      INPUT:          none
      OUTPUT:         0 if condition is not met, number neq 0 otherwise
      (SIDE)EFFECTS:  none
*/
{
  return ( ((n_tours >= max_tours) && (elapsed_time( VIRTUALT ) >= max_time)) ); 
}

void construct_solutions()
/*    
      FUNCTION:       manage the solution construction phase
      INPUT:          none
      OUTPUT:         none
      (SIDE)EFFECTS:  
*/
{
  long int k;        /* counter variable for ants */
  long int j;        /* counter variable */
  long int *objects; 
  double help, help_more;     /* bad hack to output best so far solutions earlier */ /* THOMAS */

  TRACE ( printf("construct solutions for all ants\n"); );
  help = best_so_far_ant->tour_length;
  help_more = best_so_far_ant->tour_length;

  for ( k = 0 ; k < n_ants ; k++ ) {
    n_tours++;
    TRACE ( printf("construct solution for ant %ld\n",k); );
    /* free ants for next construction */
    ant_empty_memory( &ant[k] );
    /* construct for one ant after another (sequential) */
    /* random order of objects to be assigned to locations */
    objects = generate_random_permutation( n );
    TRACE ( printf("construct solution for ant %ld\n",k); );
    for ( j = 0 ; j < n ; j++ ) {
      choose_location( &ant[k] , objects[j] );
      if ( acs_flag )
	     local_acs_pheromone_update( &ant[k], j );
    }

    TRACE ( printf("construct solution for ant %ld\n",k); );

    ant[k].tour_length = surrogate_tour_length(ant[k].tour, "ei");

    printTour(ant[k].tour);
    //getchar();
    checkTour( ant[k].tour );

    /* THOMAS: VERY VERY BAD HACK; UPDATE LATER OR SIMPLY REMOVE; ASK THOMAS IN CASE OF DOUBT */
    if ( ant[k].tour_length < help ) {
      cout << "FOUND!"<< endl;
      help = ant[k].tour_length; 
      best_so_far_ant->tour_length = help;      /* set tour length for writing report */
      write_report(  );
      best_so_far_ant->tour_length = help_more; /* set back to initial value */
    }else{
      cout << "not good"<<  ant[k].tour_length << "this" << help << endl;
    }

    free(objects);
  }
  VERYVERBOSE ( printf("end construction\n"); )

    /* THOMAS */
    /* n_tours += n_ants; */

}




void init_again_pheromone_trails( double EI)
{
  if ( !(acs_flag || mmas_flag || bwas_flag) ) {
    //trail_0 = 1. / ( (rho) * nn_tour(R) );
    trail_0 = (evaluations(0)/EI) / (rho);

    /* in the original papers on Ant System, Elitist Ant System, and
       Rank-based Ant System it is not exactly defined what the
       initial value of the pheromones is. Here we set it to some
       small constant, analogously as done in MAX-MIN Ant System.  
    */
    init_pheromone_trails( trail_0 );
  } 
  if ( bwas_flag ) {
    //trail_0 = 1. / ( (double) n * (double) nn_tour(R)) ;
    trail_0 = (evaluations(0)/EI) / ( (double) n ) ;
    init_pheromone_trails( trail_0 );
  } 
  if ( mmas_flag ) {
    //trail_max = 1. / ( (rho) * nn_tour(R) );
    //trail_min = trail_max / ( 2. * n );
    trail_max = (evaluations(0)/EI) / rho;
    trail_min = trail_max / ( 2. * n );
    init_pheromone_trails( trail_max );   
  }
  if ( acs_flag ) {
    // trail_0 = 1. / ( (double) n * (double) nn_tour(R) ) ;
    trail_0 = (evaluations(0)/EI) / ((double) n);
    init_pheromone_trails( trail_0 );
  }

}



void init_try( long int ntry ) 
/*    
      FUNCTION: initilialize variables appropriately when starting a trial
      INPUT:    trial number
      OUTPUT:   none
      COMMENTS: none
*/
{

  TRACE ( printf("INITIALIZE TRIAL\n"); );

  start_timers();
  time_used = elapsed_time( VIRTUALT );
  time_passed = time_used;

  if (comp_report) {
    fprintf(comp_report,"seed %ld\n",seed);
    fflush(comp_report);
  }

  //adapt_parameters_init();

  /* Initialize variables concerning statistics etc. */

  n_tours      = 1;
  iteration    = 1;
  restart_iteration = 1;
  lambda       = 0.05;            
  best_so_far_ant->tour_length = std::numeric_limits<double>::max();
  restart_best_ant->tour_length = std::numeric_limits<double>::max();
  found_best   = 0;

  /* Initialize the Pheromone trails, only if ACS is used, pheromones
     have to be initialized differently */
  if ( !(acs_flag || mmas_flag || bwas_flag) ) {
    //trail_0 = 1. / ( (rho) * nn_tour(R) );
    trail_0 = 1.0;
    /* in the original papers on Ant System, Elitist Ant System, and
       Rank-based Ant System it is not exactly defined what the
       initial value of the pheromones is. Here we set it to some
       small constant, analogously as done in MAX-MIN Ant System.  
    */
    init_pheromone_trails( trail_0 );
  } 
  if ( bwas_flag ) {
    //trail_0 = 1. / ( (double) n * (double) nn_tour(R)) ;
    trail_0 = 1.0;
    init_pheromone_trails( trail_0 );
  } 
  if ( mmas_flag ) {
    //trail_max = 1. / ( (rho) * nn_tour(R) );
    trail_max = 1.0;
    trail_min = trail_max / ( 2. * n );
    init_pheromone_trails( trail_max );   
  }
  if ( acs_flag ) {
    //trail_0 = 1. / ( (double) n * (double) nn_tour(R) ) ;
    trail_0 = 1.0;
    init_pheromone_trails( trail_0 );
  }

  /* Calculate combined information pheromone times heuristic information */
  compute_total_information();

  if (comp_report) fprintf(comp_report,"begin try %li \n",ntry);
  if (stat_report) fprintf(stat_report,"begin try %li \n",ntry);
}



void local_search()
/*    
      FUNCTION:       manage the local search phase; apply local search to ALL ants; in 
      dependence of ls_flag one of 2-opt, 2.5-opt, and 3-opt local search
      is chosen.
      INPUT:          none
      OUTPUT:         none
      (SIDE)EFFECTS:  all ants of the colony have locally optimal tours
      COMMENTS:       typically, best performance is obtained by applying local search 
      to all ants. It is known that some improvements (e.g. convergence 
      speed towards high quality solutions) may be obtained for some 
      ACO algorithms by applying local search to only some of the ants.
      Overall best performance is typcially obtained by using 3-opt.
*/
{
  const enum ls_types ls_type = ls_flag;
  long int k;

  //just to be sure..
  fprintf(stderr,"\n\n NO LOCAL SEARCH TO BE APPLIED IN THIS CODE!!! \n\n");
  exit(0);

  for ( k = 0 ; k < n_ants ; k++ ) {
    switch (ls_type) {
    case LS_TWO_OPT_FIRST:
      best_two_opt_qap(ant[k].tour, ant[k].tour_length);    /* 2-opt local search */
    default:
      fprintf(stderr,"type of local search procedure not correctly specified\n");
      exit(1);
    }
    ant[k].tour_length = surrogate_tour_length( ant[k].tour , "ei");
    if (termination_condition()) return;
  }
}



void update_statistics( void )
/*    
      FUNCTION:       manage some statistical information about the trial, especially
      if a new best solution (best-so-far or restart-best) is found and
      adjust some parameters if a new best solution is found
      INPUT:          none
      OUTPUT:         none
      (SIDE)EFFECTS:  restart-best and best-so-far ant may be updated; trail_min 
      and trail_max used by MMAS may be updated
*/
{

  long int iteration_best_ant;
  double p_x; /* only used by MMAS */

  iteration_best_ant = find_best(); /* iteration_best_ant is a global variable */

  if ( ant[iteration_best_ant].tour_length < best_so_far_ant->tour_length ) {

    time_used = elapsed_time( VIRTUALT ); /* best sol found after time_used */
    copy_from_to( &ant[iteration_best_ant], best_so_far_ant );
    copy_from_to( &ant[iteration_best_ant], restart_best_ant );

    found_best = iteration;
    restart_found_best = iteration;
    found_branching = node_branching(lambda);
    branching_factor = found_branching;
    if ( mmas_flag ) {
      if ( !ls_flag ) {
	p_x = exp(log(0.05)/n); 
	trail_min = 1. * (1. - p_x) / (p_x * (double)((nn_ants + 1) / 2));
	trail_max = 1. / ( (rho) * best_so_far_ant->tour_length );
	trail_0 = trail_max;
	trail_min = trail_max * trail_min; 
      } else {
	trail_max = 1. / ( (rho) * best_so_far_ant->tour_length );
	trail_min = trail_max / ( 2. * n );
	trail_0 = trail_max;
      }
      DEBUG ( printf("trail_min %.15lf trail_max %.15lf \n ",trail_min,trail_max); )
	}
    if (time_used < 1.) time_used = 1.0;
    write_report();
  }
  if ( ant[iteration_best_ant].tour_length < restart_best_ant->tour_length ) {
    copy_from_to( &ant[iteration_best_ant], restart_best_ant );
    restart_found_best = iteration;
    printf("restart best: %f, restart_found_best %ld, time %.2f\n",restart_best_ant->tour_length, restart_found_best, elapsed_time ( VIRTUALT )); 
  }
}

void update_restart(){
  if ( best_so_far_ant->tour_length < restart_best_ant->tour_length ) {
    copy_from_to( best_so_far_ant, restart_best_ant );
    restart_found_best = iteration;
    printf("restart best: %f, restart_found_best %ld, time %.2f\n",restart_best_ant->tour_length, restart_found_best, elapsed_time ( VIRTUALT )); 
  }
}



void search_control_and_statistics( void )
/*    
      FUNCTION:       occasionally compute some statistics and check whether algorithm 
      is converged 
      INPUT:          none
      OUTPUT:         none
      (SIDE)EFFECTS:  restart-best and best-so-far ant may be updated; trail_min 
      and trail_max used by MMAS may be updated
*/
{
  TRACE ( printf("SEARCH CONTROL AND STATISTICS\n"); );

  if (!(iteration % 100)) {
    population_statistics();
    branching_factor = node_branching(lambda);
    printf("\nbest so far %f, iteration: %ld, time %.2f, b_fac %.5f\n",best_so_far_ant->tour_length,iteration,elapsed_time( VIRTUALT),branching_factor);

    if ( mmas_flag && (branching_factor < branch_fac) && (iteration - restart_found_best > 250) ) {
      /* MAX-MIN Ant System was the first ACO algorithm to use
	 pheromone trail re-initialisation as implemented
	 here. Other ACO algorithms may also profit from this mechanism.
      */
      printf("INIT TRAILS!!!\n"); 
      restart_best_ant->tour_length = std::numeric_limits<double>::max(); 
      init_pheromone_trails( trail_max );
      compute_total_information();
      restart_iteration = iteration;
      restart_time = elapsed_time( VIRTUALT );
    }
    printf("try %li, iteration %li, b-fac %f \n\n", n_try,iteration,branching_factor);
  }
}



void as_update( void )
/*    
      FUNCTION:       manage global pheromone deposit for Ant System
      INPUT:          none
      OUTPUT:         none
      (SIDE)EFFECTS:  all ants deposit pheromones on matrix "pheromone"
*/
{
  long int   k;

  TRACE ( printf("Ant System pheromone deposit\n"); );

  for ( k = 0 ; k < n_ants ; k++ )
    global_update_pheromone( &ant[k] );
}



void eas_update( void )
/*    
      FUNCTION:       manage global pheromone deposit for Elitist Ant System
      INPUT:          none
      OUTPUT:         none
      (SIDE)EFFECTS:  all ants plus elitist ant deposit pheromones on matrix "pheromone"
*/
{
  long int   k;

  TRACE ( printf("Elitist Ant System pheromone deposit\n"); );

  for ( k = 0 ; k < n_ants ; k++ )
    global_update_pheromone( &ant[k] );
  global_update_pheromone_weighted( best_so_far_ant, elitist_ants );
}



void ras_update( void )
/*    
      FUNCTION:       manage global pheromone deposit for Rank-based Ant System
      INPUT:          none
      OUTPUT:         none
      (SIDE)EFFECTS:  the ras_ranks-1 best ants plus the best-so-far ant deposit pheromone 
      on matrix "pheromone"
      COMMENTS:       this procedure could be implemented slightly faster, but it is 
      anyway not critical w.r.t. CPU time given that ras_ranks is 
      typically very small.
*/
{
  long int i, k, b, target;
  double *help_b;

  TRACE ( printf("Rank-based Ant System pheromone deposit\n"); );

  help_b =(double * ) malloc( n_ants  * sizeof(double) );
  for ( k = 0 ; k < n_ants ; k++ )
    help_b[k] = ant[k].tour_length;

  for ( i = 0 ; i < ras_ranks-1 ; i++ ) {
    b = help_b[0]; target = 0;
    for ( k = 0 ; k < n_ants ; k++ ) {
      if ( help_b[k] < b ) {
	b = help_b[k]; target = k;
      }
    }
    if (target == -1 )  {
      /* printf("No Ranks larger than colony size\n"); */
      break;
    }
    help_b[target] = LONG_MAX;
    global_update_pheromone_weighted( &ant[target], ras_ranks-i-1 );
  }
  global_update_pheromone_weighted( best_so_far_ant, ras_ranks );
  free ( help_b );
}



void mmas_update( void )
/*    
      FUNCTION:       manage global pheromone deposit for MAX-MIN Ant System
      INPUT:          none
      OUTPUT:         none
      (SIDE)EFFECTS:  either the iteration-best or the best-so-far ant deposit pheromone 
      on matrix "pheromone"
*/
{
  /* we use default upper pheromone trail limit for MMAS and hence we
     do not have to worry regarding keeping the upper limit */

  long int iteration_best_ant;

  TRACE ( printf("MAX-MIN Ant System pheromone deposit\n"); );

  if ( iteration % u_gb ) {
    iteration_best_ant = find_best();
    global_update_pheromone( &ant[iteration_best_ant] );
  }
  else {
    if ( u_gb == 1 && (iteration - restart_found_best > 50))
      global_update_pheromone( best_so_far_ant );
    else 
      global_update_pheromone( restart_best_ant );
  }

  if ( ls_flag ) {
    /* implement the schedule for u_gb as defined in the 
       Future Generation Computer Systems article or in Stuetzle's PhD thesis.
       This schedule is only applied if local search is used.
    */
    if ( ( iteration - restart_iteration ) < 25 )
      u_gb = 25;
    else if ( (iteration - restart_iteration) < 75 )
      u_gb = 5;
    else if ( (iteration - restart_iteration) < 125 )
      u_gb = 3;
    else if ( (iteration - restart_iteration) < 250 )
      u_gb = 2;
    else 
      u_gb = 1;
  } else
    u_gb = 25;

}



void bwas_update( void )
/*    
      FUNCTION:       manage global pheromone deposit for Best-Worst Ant System
      INPUT:          none
      OUTPUT:         none
      (SIDE)EFFECTS:  either the iteration-best or the best-so-far ant deposit pheromone 
      on matrix "pheromone"
*/
{
  long int   iteration_worst_ant, distance_best_worst;

  TRACE ( printf("Best-worst Ant System pheromone deposit\n"); );

  global_update_pheromone( best_so_far_ant );
  iteration_worst_ant = find_worst();
  bwas_worst_ant_update( &ant[iteration_worst_ant], best_so_far_ant );
  distance_best_worst = distance_between_ants( best_so_far_ant, &ant[iteration_worst_ant]);
  /*    printf("distance_best_worst %ld, tour length worst %ld\n",distance_best_worst,ant[iteration_worst_ant].tour_length); */
  if ( distance_best_worst < (long int) (0.05 * (double) n) ) {
    restart_best_ant->tour_length = std::numeric_limits<double>::max();
    init_pheromone_trails( trail_0 );
    restart_iteration = iteration;    
    restart_time = elapsed_time( VIRTUALT );
    printf("init pheromone trails with %.15f, iteration %ld\n",trail_0,iteration);
  }
  else 
    bwas_pheromone_mutation();
}



void acs_global_update( void )
/*    
      FUNCTION:       manage global pheromone deposit for Ant Colony System
      INPUT:          none
      OUTPUT:         none
      (SIDE)EFFECTS:  the best-so-far ant deposits pheromone on matrix "pheromone"
      COMMENTS:       global pheromone deposit in ACS is done per default using 
      the best-so-far ant; Gambardella & Dorigo examined also iteration-best
      update (see their IEEE Trans. on Evolutionary Computation article), 
      but did not use it for the published computational results.
*/
{
  TRACE ( printf("Ant colony System global pheromone deposit\n"); );

  global_acs_pheromone_update( best_so_far_ant );
}



void pheromone_trail_update( void )  
/*    
      FUNCTION:       manage global pheromone trail update for the ACO algorithms
      INPUT:          none
      OUTPUT:         none
      (SIDE)EFFECTS:  pheromone trails are evaporated and pheromones are deposited 
      according to the rules defined by the various ACO algorithms.
*/
{
  /* Simulate the pheromone evaporation of all pheromones; this is not necessary 
     for ACS (see also ACO Book) */
  if ( as_flag || eas_flag || ras_flag || bwas_flag || mmas_flag ) {
    evaporation();
  }


  /* Next, apply the pheromone deposit for the various ACO algorithms */
  if ( as_flag )
    as_update(); 
  else if ( eas_flag )
    eas_update();
  else if ( ras_flag )
    ras_update();
  else if ( mmas_flag )
    mmas_update();
  else if ( bwas_flag )
    bwas_update();
  else if ( acs_flag )
    acs_global_update();

  /* check pheromone trail limits for MMAS; not necessary if local
     search is used, because in the local search case lower pheromone trail
     limits are checked in procedure mmas_evaporation_nn_list */
  if ( mmas_flag && !ls_flag )
    check_pheromone_trail_limits();

  /* Compute combined information pheromone times heuristic info after
     the pheromone update for all ACO algorithms except ACS; in the ACS case 
     this is already done in the pheromone update procedures of ACS */
  if ( as_flag || eas_flag || ras_flag || mmas_flag || bwas_flag ) {
    compute_total_information();
  }
}

template <class TYPE> std::string Str( const TYPE & t ) {
  std::ostringstream os;
  os << t;
  return os.str();
}

VectorXl generate_permutation( long int n )
/*    
      FUNCTION:       generate a random permutation of the integers 0 .. n-1
      INPUT:          length of the array
      OUTPUT:         pointer to the random permutation
      (SIDE)EFFECTS:  the array holding the random permutation is allocated in this 
      function. Don't forget to free again the memory!
      COMMENTS:       only needed by the local search procedures
*/
{
  long int  i, help, node, tot_assigned = 0;
  double    rnd;
  VectorXl r(n);

  for ( i = 0 ; i < n; i++) 
    r(i) = i;

  for ( i = 0 ; i < n ; i++ ) {
    /* find (randomly) an index for a free unit */ 
    rnd  = ran01 ( &seed );
    node = (long int) (rnd  * (n - tot_assigned)); 
    assert( i + node < n );
    help = r(i);
    r(i) = r(i+node);
    r(i+node) = help;
    tot_assigned++;
  }

  return r;
}

void create_initial_solutions(MatrixXl &matrix){

  int n_sol=matrix.rows();
  int n_col=matrix.cols();

  VectorXl aux(n_col);
  long int evaluation;

  matrix.row(0) = generate_permutation(n);
  evaluations(0) = compute_tour_length(matrix.row(0));

  bool added=FALSE;
  //solutions to generate
  for(int i=1; i < n_sol ; i++){ 
    aux = generate_permutation(n);
    evaluation = compute_tour_length(aux);
    added = FALSE;
    for(int j=0; j< n_sol && !added ; j++){
      if(evaluations(j)==-1 || evaluation < evaluations(j)){
	//not in the last element
	if(evaluations(j)!=-1 && j < (n_sol-1) ){
	  matrix.block((j+1), 0 , (n_sol-j-1), n_col) = matrix.block(j,0,(n_sol-j-1), n_col).eval();
	  evaluations.segment((j+1), (n_sol-j-1)) = evaluations.segment(j, (n_sol-j-1)).eval();
	}
	matrix.row(j)   = aux;
	evaluations(j)  = evaluation;
	added=TRUE;
      }
    }

  }
}

void remove_solution(long int del_index){
  //Remove the solution not selected.
  removeRow(solutions, del_index);
  removeElement(evaluations, del_index);
  data.evaluations = evaluations;
  removeDistance(del_index);

}

//Output signals if a solution was added or not ;)
long int add_new_solution(long int * solution, long int evaluation){

  long int n_row=solutions.rows(), add_idx=-1;

  //cout << "Adding new solution.\n\n";
  //Add new solution ordered!
  bool added=FALSE, equal=TRUE;
  for(int i=0; i<solutions.rows() && !(added); i++){
    //Do not accept duplicates!!
    if(evaluations(i)==evaluation){
      equal=TRUE;
      for(int j=0; j < solutions.cols() && equal; j++) 
	if(solutions(i,j) != solution[j]) equal=FALSE;
      if(equal){
	added=TRUE;
	std::cout << "Not adding solution, duplicate.\n";
	return(add_idx);
      }
    }
    //Add in orderupdate_restart(ant_struct* new_sol)
    if(evaluations(i) > evaluation && !added){
      addRow(solutions, solution, i);
      addElement(evaluations, evaluation, i);
      calculateDistances(FALSE, i);
      added = TRUE;
      add_idx = i;
    }

  }
  if(!added){
    addRow(solutions, solution, n_row);
    addElement(evaluations, evaluation, n_row);
    calculateDistances(FALSE, n_row);
    added = TRUE;
    add_idx = n_row;
  }


  //Reduce in case of need!!
  if(egomax < solutions.rows()){
    std::cout << "Reducing model.\n\n";
    n_row = solutions.rows();
    vector<int> search_id;
    vector<int> sel_index;

    //Initialize index to search
    for(int i=1; i<n_row ; i++){ 
      search_id.push_back(i);
    }
    VectorXd sum_dist = VectorXd::Zero(n_row);
    VectorXd mean_dist;  

    //Intialize selected: First selected is the best 
    //This is dome because is not needed to recalculate because nothing changed!
    sel_index.push_back(0);
    sum_dist = sum_dist + data.distance_matrix.col(0);

    double min_dist=0.95;
    added=FALSE;
    for(int i=1; i< egomax ; i++){
      min_dist=0.95;
      added=FALSE;
      mean_dist= sum_dist / (double) sel_index.size();
      while(!added){
	for(unsigned int j=0; j< search_id.size() && !added ; j++){
	  //Check if selected!
	  if(mean_dist(search_id[j]) >= min_dist ){
	    sel_index.push_back(search_id[j]);
	    search_id.erase (search_id.begin()+j);
	    sum_dist = sum_dist + data.distance_matrix.col(search_id[j]);
	    added=TRUE;

	  }
	}
	min_dist = min_dist - 0.1;
      }
    }

    //Remove the solution not selected.
    remove_solution(search_id[0]);

    if(search_id[0] < add_idx) add_idx--;
    if(search_id[0] == add_idx) add_idx=-1;

  }
  return(add_idx);

}

double getAverageDistance(long int * new_solution){

  double aux_dist=0.0;
  double total_dist=0.0;

  long int ncities=solutions.cols();
  for(int i=0; i < solutions.rows();i++){
    aux_dist=0.0;
    for(int j=0; j < ncities; j++){
      if(solutions(i,j) != new_solution[j] ) 
	aux_dist=aux_dist + 1.0;
    }
    aux_dist= aux_dist / (double) ncities;
    total_dist= aux_dist + total_dist;
  }

  return(total_dist/(double)solutions.rows());

}

void printModel(bool full){
  std::cout << "Model found:\n" << "  theta: "<< theta << "\n  dmodeltheta: " << dmodeltheta << "\n  ssq: "<< ssq <<"\n  mu: " << mu << std::endl; 
  if(full){
    std::cout << "Solutions: \n" << std::endl;
    std::cout << solutions <<std::endl << std::endl;
    std::cout << "Evaluations: \n" << std::endl;
    std::cout << evaluations << std::endl << std::endl;
    std::cout << "correl" << std::endl;
    std::cout << psi << endl << std::endl;
    std::cout << "theta: " << theta << std::endl ;
    std::cout << "distance" << std::endl;
    std::cout << data.distance_matrix << std::endl<< std::endl;
    std::cout << "yon:" << std::endl;
    std::cout << yon << std::endl<< std::endl;
    std::cout << "point:" << std::endl;
    std::cout << eval_sol << std::endl<< std::endl;

  }
}


/* --- main program ------------------------------------------------------ */

int main(int argc, char *argv[]) {
  /*    
	FUNCTION:       main control for running the EGO algorithm
	INPUT:          none
	OUTPUT:         none
	(SIDE)EFFECTS:  none
	COMMENTS:       none

  */

  long int i, evaluation, add_id;   
  bool aux_flag;     
  /*Initialize program*/
  start_timers();
  init_program(argc, argv);   

  /*Create initial solutions set*/
  solutions = MatrixXl::Ones(egoinitial,n);
  evaluations = VectorXl::Ones(egoinitial) * -1;
  create_initial_solutions(solutions);

  best_solution.resize(n);
  for(int k=0; k<egoinitial; k++){
    if( best_fx > evaluations(k)){
      best_fx=evaluations(k);
      for(int r=0; r < n ; r++ ) best_solution(r)=solutions(k,r);
      best_index=k;
      best_nevaluation=egoinitial;
    }
  }   

  cout << "Solutions: \n" << endl;
  cout << solutions <<endl << endl;
  cout << "Evaluations: \n" << endl;
  cout << evaluations << endl << endl;

  /*Memory of all solutions generated*/
  all_solutions=MatrixXl::Ones(egoinitial,n+1);
  all_solutions=solutions;
  all_evaluations = VectorXl::Ones(egoinitial) * -1;
  all_evaluations = evaluations;

  /*Create kriging model*/
  calculateDistances(TRUE,0);
  if(FRLH()){
    std::cout << "Model found.\n" << std::endl; 
  }else{
    std::cout << "Error: invalid initial model found (matrix inversion not possible).\n" << std::endl;
  } 
  printModel(FALSE);

  /*print best*/
  std::cout << "EGO best iterarion 0 : " << best_fx
	    << " #evaluations: " << best_nevaluation << std::endl;
  //getchar();
  /*Start EGO*/
  for(int ego_iter=0; ego_iter < (egoevals - egoinitial); ego_iter++){
    //Remove this seed setting, is just for testing
    //seed = (long int)ego_iter;

    std::cout << "\n------------> Starting EGO iteration " << Str(ego_iter+1) 
	      << " <------------" << std::endl << std::endl;

    /*Initialize/restart ACO variables*/
    pheromone = generate_double_matrix( n, n );
    total = generate_double_matrix( n, n );
    time_used = elapsed_time( VIRTUALT );
    printf("Initialization took %.10f seconds\n",time_used);
    init_try(0);
    write_report();

    /**Execute ACO**/
    while ( !termination_condition() ) {
      construct_solutions();
      update_statistics();

      //if ( ls_flag > 0 )  local_search(R);
      if ( ls_flag > 0 ) 
	best_so_far_ant->tour_length = best_two_opt_qap(best_so_far_ant->tour, best_so_far_ant->tour_length);
      update_restart();
      if (iteration == 1) {
	init_again_pheromone_trails( best_so_far_ant->tour_length );
	printf("re-init pheromone trails with %lf\n",best_so_far_ant->tour_length);
      }
      pheromone_trail_update();
      search_control_and_statistics();
      iteration++;
      //adapt_parameters_next_iteration();
    }
    exit_try(n_try);
    std::cout << "\n------------> End EGO iteration " << Str(ego_iter+1) 
	      << " <------------" << std::endl << std::endl;
     
    /*Compute real evaluation*/
    evaluation = compute_tour_length(best_so_far_ant->tour);
    /*Record best*/
    if(best_fx > evaluation){
      std::cout << "\nNew Best in iteration " << ego_iter << " previous: "
		<< Str(best_fx) << " new: " << Str(evaluation) << std::endl << std::endl;
      best_fx=evaluation;
      for(int r=0; r < n ; r++ ) best_solution(r)=best_so_far_ant->tour[r];
      best_index= egoinitial + ego_iter +1;
      best_nevaluation= egoinitial + ego_iter +1;         
    }
    std::cout << "EGO best iteration " << ego_iter << " : " << best_fx
	      << " prediction: " << surrogate_tour_length( best_solution,  "pred")
	      << " ei: " << surrogate_tour_length( best_solution,  "ei")
	      << " #evaluations: " << best_nevaluation << std::endl << std::endl;

    /*Add to the global record of candidates if needed*/
    aux_flag=FALSE;
    for(int j=0; j < all_evaluations.size() && !aux_flag; j++ ){
      if(all_evaluations(j)==evaluation){
	aux_flag=TRUE;
	for(int k=0; k < all_solutions.cols() && aux_flag; k++){
	  if(all_solutions(j,k) == best_so_far_ant->tour[k]) aux_flag=FALSE;
	}
	if(aux_flag){
	  cout << " -> Repeated solution generated with f eval:" << evaluation << endl << endl; 

	}
      }
    }

    if(!aux_flag){
      addRow(all_solutions, best_so_far_ant->tour, all_solutions.rows()-1);
      addElement(all_evaluations, evaluation, all_evaluations.size()-1);
    }
    std::cout << "EGO average diversity iteration " << ego_iter << " : " << getAverageDistance(best_so_far_ant->tour)
	      << endl << endl;

    /*Update EGO model*/
    add_id = add_new_solution(best_so_far_ant->tour , evaluation);
    if(add_id < 0){
      std::cout << "No solution added to model.\n" << std::endl;
    }else{
      if(FRLH()){
	std::cout << "Model found." << std::endl;
      }else{
	std::cout << "No valid model found (matrix inversion failed). Removing candidate." << std::endl;
	remove_solution(add_id);
      }
    }
    printModel(FALSE);
    //getchar();
  }

  std::cout << "-------------------THIS IS----------------------- " <<std::endl;
  std::cout << "EGO final best : " << best_fx 
	    << " #evaluations: " << best_nevaluation << std::endl;

  exit_program();

  std::cout << "-------------------THE END----------------------- " <<std::endl;


  free( instance.distance );
  free( instance.nn_list );

  free( pheromone );
  free( total );

  free( best_in_try );
  free( best_found_at );
  free( time_best_found );
  free( time_total_run );
  for ( i = 0 ; i < n_ants ; i++ ) {
    free( ant[i].tour );
    free( ant[i].visited );
  }
  free( ant );
  free( best_so_far_ant->tour );
  free( best_so_far_ant->visited );
  free( prob_of_selection );
  return(0);
}
