

#include <stdio.h>
#include <assert.h>
#include <stdlib.h>
#include <limits.h>
#include <iostream>

#include "ls.h"
#include "InOut.h"
#include "QAP.h"
#include "ants.h"
#include "utilities.h"

enum ls_types ls_flag;      /* indicates whether and which local search is used */
long int nn_ls;            /* maximal depth of nearest neighbour lists used in the 
			      local search */ 
long int dlb_flag = TRUE;  /* flag indicating whether don't look bits are used. I recommend 
			      to always use it if local search is applied */

long int * generate_random_permutation( long int n )
/*    
      FUNCTION:       generate a random permutation of the integers 0 .. n-1
      INPUT:          length of the array
      OUTPUT:         pointer to the random permutation
      (SIDE)EFFECTS:  the array holding the random permutation is allocated in this 
      function. Don't forget to free again the memory!
      COMMENTS:       only needed by the local search procedures
*/
{
  long int  i, help, node, tot_assigned = 0;
  double    rnd;
  long int  *r;

  r = (long int *) malloc(n * sizeof(long int));  

  for ( i = 0 ; i < n; i++) 
    r[i] = i;

  for ( i = 0 ; i < n ; i++ ) {
    /* find (randomly) an index for a free unit */ 
    rnd  = ran01 ( &seed );
    node = (long int) (rnd  * (n - tot_assigned)); 
    assert( i + node < n );
    help = r[i];
    r[i] = r[i+node];
    r[i+node] = help;
    tot_assigned++;
  }
  return r;
}


double best_two_opt_qap ( long int * q , double local_eval) {
  /*    
	FUNCTION:      first improvement local search 
	INPUT:         pointer to initial assignment
	OUTPUT:        none
	(SIDE)EFFECTS: initial assignment is locally optimized
  */

  long int   improvement = TRUE, gb_imp=FALSE;
  long int   u, v, help;
  double     local_best_value, new_value;

  // local_best_value = surrogate_tour_length(q, "ei");
  local_best_value = local_eval;
      
  while ( improvement ) {
    improvement = FALSE;
    for ( u = 0 ; u < n ; u++) {
      for ( v = 0 ; v < n ; v++) {
	if (u == v)
	  continue;
	help     = q[u];
	q[u] = q[v];
	q[v] = help;    
	new_value = surrogate_tour_length(q, "ei");

	if (new_value < local_best_value) {
	  local_best_value = new_value;
	  improvement = TRUE;
          gb_imp=TRUE;
	}else {
	  /* no improvement, then undo the exchange move again */
	  help     = q[u];
	  q[u] = q[v];
	  q[v] = help;
	}

      }
    }
  }
  if(gb_imp) printf(" local seach: improvement found.\n");
  return(local_best_value);
}


